%% Lezione SAR - misure di riconoscimento/verifica
clear all;
close all;

% ROC curve - Esercizio 1: riproporre gli esempi visti sulle slide delle
% varie casistiche di curva ROC

%http://www0.cs.ucl.ac.uk/staff/ucacbbl/roc/

NEG_theta=[3,1]; % mean and std of negative class
POS_theta=[4,1]; % mean and std of positive class

nsample = 1000;
mu1 = NEG_theta(1);
sigma1 = NEG_theta(2);

mu2 = POS_theta(1);
sigma2 = POS_theta(2);

NEG_d = normrnd(mu1,sigma1,nsample,1);
POS_d = normrnd(mu2,sigma2,nsample,1);

labels = [ones(nsample,1).*-1;ones(nsample,1)];



figure; subplot(1,2,1); set(gcf,'name',strcat('Curva ROC: NEG= N(',num2str(mu1),',',num2str(sigma1),'), POS=N(',num2str(mu2),',',num2str(sigma2),')'));
pdf_NEG_d = normpdf(NEG_d,mu1,sigma1);
pdf_POS_d = normpdf(POS_d,mu2,sigma2);
[ord_N,ind] = sort(NEG_d,'ascend'); ord_PN = pdf_NEG_d(ind);
[ord_P,ind] = sort(POS_d,'ascend'); ord_PP = pdf_POS_d(ind);
plot(ord_N,ord_PN,'r'); hold on
plot(ord_P,ord_PP,'g');
title('Distribuzioni')

[X,Y,T,AUC] = perfcurve(labels,[NEG_d;POS_d],1);
subplot(1,2,2); plot(X,Y)
xlabel('False Positive Rate (FPR)')
ylabel('True Positive Rate (TPR)')
title('ROC')





% [X,Y] = perfcurve(LABELS,SCORES,POSCLASS) computes a ROC curve for a
%     vector of classifier predictions SCORES given true class labels,
%     LABELS. The labels can be a numeric vector, logical vector, character
%     matrix, cell array of strings or categorical vector (see help for
%     groupingvariable). SCORES is a numeric vector of scores returned by a
%     classifier for some data. This vector must have as many elements as
%     LABELS does. POSCLASS is the positive class label (scalar), either
%     numeric (for numeric LABELS), logical (for logical LABELS) or char. The
%     specified positive class must be in the array of input labels. The
%     returned values X and Y are coordinates for the performance curve and
%     can be visualized  with PLOT(X,Y). By default, X is false positive
%     rate, FPR, (equivalently, fallout, or 1-specificity) and Y is true
%     positive rate, TPR, (equivalently, recall, or sensitivity).
%    

% [X,Y,T] = perfcurve(LABELS,SCORES,POSCLASS) returns an array T of
%     thresholds on classifier scores for the computed values of X and Y. It
%     has the same number of rows as X and Y. For each threshold, TP is the
%     count of true positive observations with scores greater or equal to
%     this threshold, and FP is the count of false positive observations with
%     scores greater or equal to this threshold. perfcurve defines negative
%     counts, TN and FN, in a similar way and sorts the thresholds in the
%     descending order which corresponds to the ascending order of positive
%     counts. For the M distinct thresholds found in the array of scores,
%     perfcurve returns the X, Y and T arrays with M+1 rows. perfcurve
%     sets elements T(2:M+1) to the distinct thresholds, and T(1) replicates
%     T(2). By convention, T(1) represents the highest 'reject all' threshold
%     and perfcurve computes the corresponding values of X and Y for TP=0 and
%     FP=0. T(end) is the lowest 'accept all' threshold for which TN=0 and
%     FN=0.


% analisi del people detector

load('svmOut_0_HOG3_6_86_34_F.mat')
labels = [ones(length(negative),1).*-1;ones(length(positive),1)];
[X,Y,T,AUC] = perfcurve(labels,[negative;positive],1);
figure; plot(X,Y)
xlabel('False Positive Rate (FPR)')
ylabel('True Positive Rate (TPR)')
title('ROC')

% costruzione "a mano" della ROC

Nmin = min(negative);
Nmax = max(negative);
Pmin = min(positive);
Pmax = max(positive);

AbsMin = min(Nmin,Pmin);
AbsMax = max(Nmax,Pmax);

num_values = 1000; % numero valori della soglia che voglio valutare;
step = abs(AbsMax - AbsMin)/num_values
i=1;
for t=AbsMax:-step:AbsMin
    TPR(i)= sum(positive>t)/length(positive);
    FPR(i)= sum(negative>t)/length(negative);
    thresh(i) = t;
    i=i+1;
end

figure; plot(FPR,TPR)
xlabel('False Positive Rate (FPR)')
ylabel('True Positive Rate (TPR)')
title('ROC a mano')
    
% Calcolo dell'EER sulla curva fatta a mano
[minimo,ind]=min(abs(FPR-(1-TPR)));
EER = FPR(ind);
% Calcolo della soglia con cui ho l'EER
optimal_thresh = thresh(ind);

    
% Calcolo dell'EER sulla curva fatta con perfcurve
[minimo,ind]=min(abs(X-(1-Y)));
EER = X(ind)



alpha = 1 % cost of a false positive (false alarm);
beta = 100 % cost of missing a positive (false negative);
p = length(positive)/length(negative) %proportion of positive cases;

% the average expected cost of classification at point x,y in the ROC space is

C = (1-p).*alpha.*FPR + p.*beta.*(TPR) ;
[best,bop]=min(C)
figure; plot(C)

% CMC
num_p = 10; % number of probe elements
num_g = 10; % number of gallery elements
l_p = [1:num_p]; % labels of the probe identities
l_g = [1:num_g]; % labels of the gallery identities

Sim = zeros(num_p,num_g); % similarity distance



% Ideal CMC
sim_toy = [0:num_p-1]'/10; % toy similarity distances
for i=1:num_p
    Sim(i,:)=circshift(sim_toy,i-1)'; % create a dummy vector of distances
end



for p = 1:num_p
                   [~,idx] = sort(Sim(p,:),'ascend');            
                   rank_pos(p) = find(idx==l_p(p)); % find the position of the exact match in the similarity vector
end
for p = 1:num_p
                    cmc(p) = sum(rank_pos <= p);
end
cmc = cmc ./ num_p






figure; set(gcf,'name','ideal CMC');
plot(1:num_g, cmc);
xlabel('rank');
ylabel('probability of rank');

%%Esercizio 1: operando su Sim, fate in modo che il valore di cmc(1)=0.5;

for i=1:num_p/2
    Sim(i,:)=circshift(Sim(i,:)',-1)'; % scrabble the distances a bit...
end

for p = 1:num_p            
                   [~,idx] = sort(Sim(p,:),'ascend');            
                   rank_pos(p) = find(idx==l_p(p)); % find the position of the exact match in the similarity vector
end
for p = 1:num_p
                    cmc(p) = sum(rank_pos <= p);
end
cmc = cmc ./ num_p

figure; set(gcf,'name','CMC(1) = 0.5');
plot(1:num_g, cmc);

%%Esercizio 2: operando su Sim, fate in modo che il valore di cmc(1)=0.5; cmc(2)=0.8;


for i=[1,2]
    Sim(i,:)=circshift(Sim(i,:)',-1)'; % scrabble the distances a bit...
end

for p = 1:num_p            
                   [~,idx] = sort(Sim(p,:),'ascend');            
                   rank_pos(p) = find(idx==l_p(p)); % find the position of the exact match in the similarity vector
end
for p = 1:num_p
                    cmc(p) = sum(rank_pos <= p);
end
cmc = cmc ./ num_p

figure; set(gcf,'name','CMC(1) = 0.5, CMC(2) = 0.8');
plot(1:num_g, cmc);


%%Esercizio 3: operando su Sim, costruire una ROC;

%L'idea e' di prendere i valori sulla diagonale della matrice di
%similarita' come esempi positivi, gli elementi off-diagonali come negativi

